@@ -10,7 +10,7 @@ from contextlib import contextmanager, suppress
 
 from .borders import load_borders_program
 from .boss import Boss
-from .child import set_default_env
+from .child import set_default_env, openpty, remove_blocking
 from .cli import create_opts, parse_args
 from .config import cached_values_for, initial_window_size_func
 from .constants import (
@@ -203,41 +203,78 @@ def macos_cmdline(argv_args):
     return ans
 
 
-def read_shell_environment(opts):
+def read_shell_environment(opts=None):
     if not hasattr(read_shell_environment, 'ans'):
         import subprocess
         from .session import resolved_shell
         shell = resolved_shell(opts)
-        p = subprocess.Popen(shell + ['-l', '-c', 'env'], stdout=subprocess.PIPE)
-        raw = p.stdout.read()
-        if p.wait() == 0:
-            raw = raw.decode('utf-8', 'replace')
-            ans = read_shell_environment.ans = {}
-            for line in raw.splitlines():
-                k, v = line.partition('=')[::2]
-                if k and v:
-                    ans[k] = v
+        master, slave = openpty()
+        remove_blocking(master)
+        p = subprocess.Popen(shell + ['-l', '-c', 'env'], stdout=slave, stdin=slave, stderr=slave, start_new_session=True, close_fds=True)
+        with os.fdopen(master, 'rb') as stdout, os.fdopen(slave, 'wb'):
+            raw = b''
+            from subprocess import TimeoutExpired
+            from time import monotonic
+            start_time = monotonic()
+            while monotonic() - start_time < 1.5:
+                try:
+                    ret = p.wait(0.01)
+                except TimeoutExpired:
+                    ret = None
+                with suppress(Exception):
+                    raw += stdout.read()
+                if ret is not None:
+                    break
+            if p.returncode is None:
+                log_error('Timed out waiting for shell to quit while reading shell environment')
+                p.kill()
+            elif p.returncode == 0:
+                while True:
+                    try:
+                        x = stdout.read()
+                    except Exception:
+                        break
+                    if not x:
+                        break
+                    raw += x
+                raw = raw.decode('utf-8', 'replace')
+                ans = read_shell_environment.ans = {}
+                for line in raw.splitlines():
+                    k, v = line.partition('=')[::2]
+                    if k and v:
+                        ans[k] = v
+            else:
+                log_error('Failed to run shell to read its environment')
+                read_shell_environment.ans = {}
     return read_shell_environment.ans
 
 
+def get_editor_from_env(shell_env):
+    for var in ('VISUAL', 'EDITOR'):
+        editor = shell_env.get(var)
+        if editor:
+            if 'PATH' in shell_env:
+                import shlex
+                editor_cmd = shlex.split(editor)
+                if not os.path.isabs(editor_cmd[0]):
+                    editor_cmd[0] = shutil.which(editor_cmd[0], path=shell_env['PATH'])
+                    if editor_cmd[0]:
+                        editor = ' '.join(map(shlex.quote, editor_cmd))
+                    else:
+                        editor = None
+            if editor:
+                return editor
+
+
 def setup_environment(opts, args):
     extra_env = opts.env.copy()
     if opts.editor == '.':
-        if 'EDITOR' not in os.environ:
+        editor = get_editor_from_env(os.environ)
+        if not editor:
             shell_env = read_shell_environment(opts)
-            if 'EDITOR' in shell_env:
-                editor = shell_env['EDITOR']
-                if 'PATH' in shell_env:
-                    import shlex
-                    editor_cmd = shlex.split(editor)
-                    if not os.path.isabs(editor_cmd[0]):
-                        editor_cmd[0] = shutil.which(editor_cmd[0], path=shell_env['PATH'])
-                        if editor_cmd[0]:
-                            editor = ' '.join(map(shlex.quote, editor_cmd))
-                        else:
-                            editor = None
-                if editor:
-                    os.environ['EDITOR'] = editor
+            editor = get_editor_from_env(shell_env)
+        if editor:
+            os.environ['EDITOR'] = editor
     else:
         os.environ['EDITOR'] = opts.editor
     if args.listen_on:
