@@ -153,6 +153,10 @@ screen_reset(Screen *self) {
     set_dynamic_color(self, 110, NULL);
     set_dynamic_color(self, 111, NULL);
     set_color_table_color(self, 104, NULL);
+    self->parser_state = 0;
+    self->parser_text_start = 0;
+    self->parser_buf_pos = 0;
+    self->parser_has_pending_text = false;
 }
 
 void
@@ -737,6 +741,24 @@ screen_tab(Screen *self) {
     }
     if (!found) found = self->columns - 1;
     if (found != self->cursor->x) {
+        if (self->cursor->x < self->columns) {
+            linebuf_init_line(self->linebuf, self->cursor->y);
+            combining_type diff = found - self->cursor->x;
+            CPUCell *cpu_cell = self->linebuf->line->cpu_cells + self->cursor->x;
+            bool ok = true;
+            for (combining_type i = 0; i < diff; i++) {
+                CPUCell *c = cpu_cell + i;
+                if (c->ch != ' ' && c->ch != 0) { ok = false; break; }
+            }
+            if (ok) {
+                for (combining_type i = 0; i < diff; i++) {
+                    CPUCell *c = cpu_cell + i;
+                    c->ch = ' '; zero_at_ptr_count(c->cc_idx, arraysz(c->cc_idx));
+                }
+                cpu_cell->ch = '\t';
+                cpu_cell->cc_idx[0] = diff;
+            }
+        }
         self->cursor->x = found;
     }
 }
@@ -1516,7 +1538,7 @@ screen_update_cell_data(Screen *self, void *address, FONTS_DATA_HANDLE fonts_dat
 
 static inline bool
 is_selection_empty(Screen *self, unsigned int start_x, unsigned int start_y, unsigned int end_x, unsigned int end_y) {
-    return (start_x >= self->columns || start_y >= self->lines || end_x >= self->columns || end_y >= self->lines || (start_x == end_x && start_y == end_y)) ? true : false;
+    return start_x >= self->columns || start_y >= self->lines || end_x >= self->columns || end_y >= self->lines || (start_x == end_x && start_y == end_y);
 }
 
 static inline void
@@ -1708,7 +1730,7 @@ static PyObject*
 set_pending_timeout(Screen *self, PyObject *val) {
     if (!PyFloat_Check(val)) { PyErr_SetString(PyExc_TypeError, "timeout must be a float"); return NULL; }
     PyObject *ans = PyFloat_FromDouble(self->pending_mode.wait_time);
-    self->pending_mode.wait_time = PyFloat_AS_DOUBLE(val);
+    self->pending_mode.wait_time = s_double_to_monotonic_t(PyFloat_AS_DOUBLE(val));
     return ans;
 }
 
@@ -1989,6 +2011,12 @@ is_rectangle_select(Screen *self, PyObject *a UNUSED) {
     return ans;
 }
 
+static PyObject*
+copy_colors_from(Screen *self, Screen *other) {
+    copy_color_profile(self->color_profile, other->color_profile);
+    Py_RETURN_NONE;
+}
+
 static PyObject*
 text_for_selection(Screen *self, PyObject *a UNUSED) {
     FullSelectionBoundary start, end;
@@ -2239,6 +2267,13 @@ wcwidth_wrap(PyObject UNUSED *self, PyObject *chr) {
     return PyLong_FromLong(wcwidth_std(PyLong_AsLong(chr)));
 }
 
+static PyObject*
+screen_is_emoji_presentation_base(PyObject UNUSED *self, PyObject *code_) {
+    unsigned long code = PyLong_AsUnsignedLong(code_);
+    if (is_emoji_presentation_base(code)) Py_RETURN_TRUE;
+    Py_RETURN_FALSE;
+}
+
 
 #define MND(name, args) {#name, (PyCFunction)name, args, #name},
 #define MODEFUNC(name) MND(name, METH_NOARGS) MND(set_##name, METH_O)
@@ -2294,6 +2329,7 @@ static PyMethodDef methods[] = {
     MND(toggle_alt_screen, METH_NOARGS)
     MND(reset_callbacks, METH_NOARGS)
     MND(paste, METH_O)
+    MND(copy_colors_from, METH_O)
     {"select_graphic_rendition", (PyCFunction)_select_graphic_rendition, METH_VARARGS, ""},
 
     {NULL}  /* Sentinel */
@@ -2351,6 +2387,7 @@ PyTypeObject Screen_Type = {
 static PyMethodDef module_methods[] = {
     {"wcwidth", (PyCFunction)wcwidth_wrap, METH_O, ""},
     {"wcswidth", (PyCFunction)screen_wcswidth, METH_O, ""},
+    {"is_emoji_presentation_base", (PyCFunction)screen_is_emoji_presentation_base, METH_O, ""},
     {"truncate_point_for_length", (PyCFunction)screen_truncate_point_for_length, METH_VARARGS, ""},
     {NULL}  /* Sentinel */
 };
