@@ -7,6 +7,7 @@
 #pragma once
 
 #include "graphics.h"
+#include "monotonic.h"
 
 typedef enum ScrollTypes { SCROLL_LINE = -999999, SCROLL_PAGE, SCROLL_FULL } ScrollType;
 
@@ -66,6 +67,7 @@ typedef struct {
 
     unsigned int columns, lines, margin_top, margin_bottom, charset, scrolled_by, last_selection_scrolled_by;
     unsigned int last_rendered_cursor_x, last_rendered_cursor_y;
+    double pending_scroll_pixels;
     CellPixelSize cell_size;
     OverlayLine overlay_line;
     id_type window_id;
@@ -86,13 +88,13 @@ typedef struct {
     bool *tabstops, *main_tabstops, *alt_tabstops;
     ScreenModes modes;
     ColorProfile *color_profile;
-    double start_visual_bell_at;
+    monotonic_t start_visual_bell_at;
 
     uint32_t parser_buf[PARSER_BUF_SZ];
     unsigned int parser_state, parser_text_start, parser_buf_pos;
     bool parser_has_pending_text;
     uint8_t read_buf[READ_BUF_SZ], *write_buf;
-    double new_input_at;
+    monotonic_t new_input_at;
     size_t read_buf_sz, write_buf_sz, write_buf_used;
     pthread_mutex_t read_buf_lock, write_buf_lock;
 
@@ -101,7 +103,7 @@ typedef struct {
     struct {
         size_t capacity, used, stop_buf_pos;
         uint8_t *buf;
-        double activated_at, wait_time;
+        monotonic_t activated_at, wait_time;
         int state;
         uint8_t stop_buf[32];
     } pending_mode;
@@ -110,8 +112,8 @@ typedef struct {
 } Screen;
 
 
-void parse_worker(Screen *screen, PyObject *dump_callback, double now);
-void parse_worker_dump(Screen *screen, PyObject *dump_callback, double now);
+void parse_worker(Screen *screen, PyObject *dump_callback, monotonic_t now);
+void parse_worker_dump(Screen *screen, PyObject *dump_callback, monotonic_t now);
 void screen_align(Screen*);
 void screen_restore_cursor(Screen *);
 void screen_save_cursor(Screen *);
