@@ -1,7 +1,7 @@
 #pragma once
 #include "data-types.h"
-#define VS15 1281
-#define VS16 1282
+#define VS15 1280
+#define VS16 1281
 
 bool is_combining_char(char_type ch);
 bool is_ignored_char(char_type ch);
@@ -24,3 +24,8 @@ can_strip_from_end_of_url(uint32_t ch) {
         ch == '>'
     ) ? true : false;
 }
+
+static inline bool
+is_private_use(char_type ch) {
+    return (0xe000 <= ch && ch <= 0xf8ff) || (0xF0000 <= ch && ch <= 0xFFFFF) || (0x100000 <= ch && ch <= 0x10FFFF);
+}
